#import "TestService.h"
#import <GNUstepBase/NSStream+GNUstepBase.h>

@interface TestService (Private)
- (void)setup;
- (void)searchTestServices;
- (void)setupTestServiceListenerSocket;
- (void)registerTestService;
- (NSString *)serviceName;
@end

@implementation TestService

static BOOL shouldDebugNSNetService = NO;

NSString *TestServiceType = @"_test-service._tcp.";

+ (void)initialize {
    static BOOL didInit = NO;
    if (didInit)
        return;

    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    shouldDebugNSNetService = [ud boolForKey:@"DebugNSNetService"];
    didInit = YES;
}

+ (id)service {
    return [[[self alloc] init] autorelease];
}

- (id)init {
    self = [super init];
    if (self) {
        [self setup];
    }
    return self;
}

- (void)dealloc {
    [self->browser release];
    [self->services release];
    [self->service release];
    [self->serverStream release];

    [super dealloc];
}

#pragma mark -- Setup --

- (void)setup {
    [self setupTestServiceListenerSocket];
    [self registerTestService];
    [self searchTestServices];
}

- (void)setupTestServiceListenerSocket {
    self->serverStream = [[GSServerStream serverStreamToAddr:@"" port:0]
                                          retain];
    NSAssert(self->serverStream, @"Could not open random port?");
    [self->serverStream setDelegate:self];
    [self->serverStream open];
    [self->serverStream scheduleInRunLoop:[NSRunLoop currentRunLoop]
                        forMode:NSDefaultRunLoopMode];
}

- (NSString *)serviceName {
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    NSString *name = [ud stringForKey:@"ServiceName"];
    if (!name)
        name = [[NSProcessInfo processInfo] globallyUniqueString];
    return name;
}

- (void)registerTestService {
    int servicePort = (int)[[self->serverStream
                                   propertyForKey:GSStreamLocalPortKey]
                                   intValue];
    NSString *name = [self serviceName];
    NSLog(@"service:%@ type:%@ port:%d", name, TestServiceType, servicePort);
    self->service = [[NSNetService alloc] initWithDomain:@""
                                          type:TestServiceType
                                          name:name
                                          port:servicePort];
    [self->service publish];
}

- (void)searchTestServices {
    self->services = [[NSMutableArray alloc] init];
    self->browser  = [[NSNetServiceBrowser alloc] init];
    [self->browser setDelegate:self];
    [self->browser searchForServicesOfType:TestServiceType
                   inDomain:@""];
}


#pragma mark -- Runloop --

- (void)run {
    [[NSRunLoop currentRunLoop] run];
}

#pragma mark -- NSNetServiceBrowserDelegate --

- (void)netServiceBrowser:(NSNetServiceBrowser *)aNetServiceBrowser
  didFindService:(NSNetService *)_service
  moreComing:(BOOL)moreComing
{
    if (shouldDebugNSNetService)
        NSLog(@"%s service:%@", __PRETTY_FUNCTION__, _service);

    NSLog(@"Found TestService '%@', resolving.", [_service name]);
    [_service retain];
    [_service setDelegate:self];
    [_service resolveWithTimeout:15.0];
}

- (void)netServiceBrowser:(NSNetServiceBrowser *)aNetServiceBrowser
  didRemoveService:(NSNetService *)_service
  moreComing:(BOOL)moreComing
{
    if (shouldDebugNSNetService)
        NSLog(@"%s service:%@", __PRETTY_FUNCTION__, _service);

    NSLog(@"Removed TestService '%@'", [_service name]);
    [self->services removeObject:_service];
}

- (void)netServiceBrowserWillSearch:(NSNetServiceBrowser *)aNetServiceBrowser {
    NSLog(@"Searching for TestServices...");
}

- (void)netServiceBrowserDidStopSearch:(NSNetServiceBrowser *)aNetServiceBrowser {
    NSLog(@"Stop searching for TestServices");
}

#pragma mark -- NSNetServiceDelegate --

- (void)netServiceDidResolveAddress:(NSNetService *)_service {
    if (shouldDebugNSNetService)
        NSLog(@"%s service:%@", __PRETTY_FUNCTION__, _service);

    if (![self->services containsObject:_service])
    {
        [self->services addObject:_service];
        [_service release]; // NOTE: retained this for resolving
        NSLog(@"Added TestService '%@'", [_service name]);
    }
}

@end
